# -*- coding: utf8 -*-

from __future__ import unicode_literals

import logging
from xml.etree import cElementTree as ET

import six

from driverng import MessageV2, Worklist
from driverng.exceptions import ParsingError
from driverng.utils import format_date, get_date

logger = logging.getLogger("driverng.cytovision")


class Driver(MessageV2):
    SPEC = {"code": "cytovision", "label": "cytovision XML"}
    ENCODING = "utf-8"

    def parse(self, data):
        xml = data.encode("utf8")
        # Remove UFT-8 BOM
        if not xml.startswith(b"<?xml"):
            xml = xml.decode("utf-8-sig")
            xml = xml.encode("utf8")
        # Suppression des caractères indesirables du début de "data"
        try:
            if six.PY3:
                deb = xml.decode().index("<?xml")
            else:
                deb = xml.index("<?xml")
        except ValueError:
            deb = 0

        if deb > 0:
            xml = xml[deb:]
        try:
            return ET.fromstring(xml)
        except ET.ParseError:
            logger.exception("XML parsing error")
            raise ParsingError(xml[:10])

    def to_string(self, message):
        header = '<?xml version="1.0" encoding="UTF-8"?>'
        return header + ET.tostring(message).decode()

    def setCasedetail(self, casedetailset, detail, valeur, ctrltype="Text"):
        casedetail = ET.SubElement(casedetailset, "CaseDetail")
        casedetail.set("ctrltype", ctrltype)
        casedetail.set("editable", "True")
        casedetail.set("mandatory", "False")
        casedetail.set("text", valeur)
        casedetail.set("title", detail)
        casedetail.set("type", "system")

    def encode_worklist(self, worklist, context):
        root = ET.Element("CWS")
        n_order = 0
        for sample in worklist.samples:
            n_order += 1
            case = ET.SubElement(root, "Case")
            case.set("name", sample.sample_id)
            casedetail = ET.SubElement(case, "CaseDetailsSet")
            self.setCasedetail(
                casedetail,
                "Date du dossier",
                format_date(sample.collection_date, fmt="%d/%m/%Y"),
                ctrltype="Date",
            )
            self.setCasedetail(casedetail, "Nom", sample.last_name)
            self.setCasedetail(casedetail, "Nom de naissance", sample.maiden_name)
            self.setCasedetail(casedetail, "Prenom", sample.first_name)
            self.setCasedetail(
                casedetail,
                "Date de naissance",
                format_date(sample.birthdate, fmt="%d/%m/%Y"),
                ctrltype="Date",
            )
            order_codes = {}
            for rg, order in enumerate(sample.orders):
                # Les suppressions ne sont pas gérées
                if order.action == "del":
                    continue
                codes = order.code.split(",")
                valeur = ""
                value_label = ""
                if len(codes) > 1:
                    ctype = codes[1]
                    if ctype == "Number":
                        valeur = "0"
                else:
                    ctype = "Text"
                try:
                    rang = int(codes[2])
                except Exception:
                    rang = rg + 1

                vals = [
                    r for r in sample.results if not r.anterior and r.code == order.code
                ]
                if vals and vals[0]:
                    valeur = ctype == "Text" and vals[0].value_label or vals[0].value
                    value_label = vals[0].value_label
                order_codes[codes[0]] = {
                    "code": codes[0],
                    "valeur": valeur,
                    "value_label": value_label,
                    "ctrltype": ctype,
                    "rang": rang,
                }
                n_order += 1
            for r in [
                r
                for r in sample.results
                if not r.anterior and r.code not in order_codes.keys()
            ]:
                codes = r.code.split(",")
                if len(codes) > 1:
                    ctype = codes[1]
                else:
                    ctype = "Text"
                try:
                    rang = int(codes[2])
                except Exception:
                    rang = 0
                order_codes[codes[0]] = {
                    "code": codes[0],
                    "valeur": r.value,
                    "value_label": r.value_label,
                    "ctrltype": ctype,
                    "rang": rang,
                }
                n_order += 1
            for key, order in sorted(order_codes.items(), key=lambda y: y[1]["rang"]):
                value = order["valeur"]
                if order["ctrltype"] == "Date" and order["valeur"]:
                    date = get_date(order["valeur"])
                    if date:
                        value = format_date(date, fmt="%d/%m/%Y")
                if order["ctrltype"] == "Text" and order["value_label"]:
                    value = order["value_label"]
                self.setCasedetail(
                    casedetail, key, valeur=value, ctrltype=order["ctrltype"]
                )
        if n_order:
            return root

    def decode_message(self, message):
        worklist = Worklist(context="result")
        cws = message.tag
        if cws == "CWS":
            case = message.find("Case")
            if case is None:
                return worklist
            sample_id = case.get("name")
            sample = worklist.get_or_create(sample_id=sample_id, type="patient")
            for detail in case.find("CaseDetailsSet").findall("CaseDetail"):
                test = detail.get("title").strip()
                if test not in [
                    "Nom",
                    "Nom de naissance",
                    "Prenom",
                    "Date de naissance",
                ]:
                    value = detail.get("text").strip()
                    if value:
                        sample.results.add(code=test, value=value)
            flag = case.find("Flag")
            resu = None
            if flag is not None:
                resu = flag.get("title").strip()
                if resu:
                    sample.results.add(code="Flag", value=resu.replace("\u2019", "'"))

        return worklist
